'use strict';

const noop = () => {};

const Errors = (() => {
  const _errors = [];
  return {
    getStatus: () => _errors.length && _errors.join('\n'),
    logError: logError,
    createDatabaseErrorHandler: createDatabaseErrorHandler
  };
  function logError(message, data) {
    _errors.push(`${message}${data ? `:\n${JSON.stringify(data || '')}` : ''}`);
  }
  function createDatabaseErrorHandler(message, rejectFn = noop) { return e => {
    console.log(message, e);
    logError(message);
    logError(get(e, 'target.error.name'), get(e, 'target.error.message'));
    rejectFn(e)
  }}
})(); // Errors

function MAIN_INIT() {
  browser.tabs.create({url: 'dial.html', active: true})
}

browser.runtime.onMessage.addListener((data, sender) => {
  console.log('BACKGROUND received message: ', data, sender);
  return DB.getThumbnails(0).then(message => `db errors: ${Errors.getStatus()}, result message: ${message}`);
});

const DB_OP_READWRITE = 'readwrite', DB_OP_READONLY = 'readonly';
const DB_IMAGE = 'image', DB_HISTORY = 'history', DB_ORIGINAL = 'original', DB_PREVIOUS = 'previous', GROUP_INDEX = 'group';
const DB = (() => {   // IndexedDB
  const DB_VERSION = 6, DB_NAME = 'group_speed_dial';
  const _dbPromise = initDB();
  return {
    getThumbnails: getThumbnails
  };
  function initDB() { return new Promise((resolve, reject) => {
    const DBOpenRequest = window.indexedDB.open(DB_NAME, DB_VERSION);
    // This event handles the event whereby a new version of the database needs to be created
    DBOpenRequest.onupgradeneeded = e => {
      console.log('DB: onupgradeneeded');
      const db = e.target.result;
      db.onerror = Errors.createDatabaseErrorHandler('error db creation', reject);
      if(!db.objectStoreNames.contains(DB_IMAGE)) db.createObjectStore(DB_IMAGE, { keyPath: ['url', 'group'] });
      if(!db.objectStoreNames.contains(DB_HISTORY)) db.createObjectStore(DB_HISTORY, { keyPath: 'id', autoIncrement: true });
      if(!db.objectStoreNames.contains(DB_ORIGINAL)) db.createObjectStore(DB_ORIGINAL, { keyPath: ['url', 'group'] });
      if(!db.objectStoreNames.contains(DB_PREVIOUS)) db.createObjectStore(DB_PREVIOUS, { keyPath: ['url', 'group'] });
      // create new indexes
      const store = e.target.transaction.objectStore(DB_IMAGE);
      if(!store.indexNames.contains(GROUP_INDEX)) store.createIndex(GROUP_INDEX, GROUP_INDEX, { unique: false });
    };
    DBOpenRequest.onerror = Errors.createDatabaseErrorHandler('error db access', reject);
    DBOpenRequest.onsuccess = e => {
      console.log('DB: onsuccess');
      resolve(DBOpenRequest.result);
    };
  })}

  async function getThumbnails(groupId) {
    const db = await _dbPromise;
    const getAllRequest = db.transaction(DB_IMAGE, DB_OP_READONLY).objectStore(DB_IMAGE).index(GROUP_INDEX).getAll(groupId);
    return new Promise((resolve, reject) => {
      getAllRequest.onerrror = reject;
      getAllRequest.onsuccess = e => {
        resolve('retrieval OK');
      };
    });
  }

})(); // DB

// run the app!
MAIN_INIT();
